@@ -1,7 +1,7 @@
 /*
 ** options.c - oidentd command-line handler.
 ** Copyright (c) 2001-2006 Ryan McCabe <ryan@numb.org>
-** Copyright (c) 2018-2019 Janik Rabe  <oidentd@janikrabe.com>
+** Copyright (c) 2018-2019 Janik Rabe  <info@janikrabe.com>
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License, version 2,
@@ -41,14 +41,15 @@
 #include "options.h"
 
 #if MASQ_SUPPORT
-#	define OPTSTRING "a:c:C:def::g:hiIl:mMo::p:P:qr:St:u:Uv"
+#	define OPTSTRING "a:c:C:def::g:hiIl:mMo::p:P:qr:R:St:u:Uv"
 	extern in_port_t fwdport;
 #else
-#	define OPTSTRING "a:c:C:deg:hiIl:o::p:P:qr:St:u:Uv"
+#	define OPTSTRING "a:c:C:deg:hiIl:o::p:P:qr:R:St:u:Uv"
 #endif
 
 extern struct sockaddr_storage proxy;
 extern char *failuser;
+extern char *replyall;
 extern char *ret_os;
 extern char *config_file;
 extern u_int32_t timeout;
@@ -65,34 +66,31 @@ static void print_version(bool verbose);
 static inline void enable_opt(u_int32_t option);
 
 static const struct option longopts[] = {
-	{"address",				required_argument,	0, 'a'},
-	{"charset",				required_argument,	0, 'c'},
-	{"config",				required_argument,	0, 'C'},
-	{"debug",				no_argument,		0, 'd'},
-	{"error",				no_argument,		0, 'e'},
-	{"group",				required_argument,	0, 'g'},
-	{"help",				no_argument,		0, 'h'},
-	{"foreground",				no_argument,		0, 'i'},
-	{"stdio",				no_argument,		0, 'I'},
-	{"limit",				required_argument,	0, 'l'},
-	{"other",				optional_argument,	0, 'o'},
-	{"port",				required_argument,	0, 'p'},
-	{"quiet",				no_argument,		0, 'q'},
-	{"reply",				required_argument,	0, 'r'},
-	{"nosyslog",				no_argument,		0, 'S'},
-	{"timeout",				required_argument,	0, 't'},
-	{"user",				required_argument,	0, 'u'},
-#if HAVE_LIBUDB
-	{"udb",					no_argument,		0, 'U'},
-#endif
-	{"version",				no_argument,		0, 'v'},
+	{"address",          required_argument, 0, 'a'},
+	{"charset",          required_argument, 0, 'c'},
+	{"config",           required_argument, 0, 'C'},
+	{"debug",            no_argument,       0, 'd'},
+	{"error",            no_argument,       0, 'e'},
+	{"group",            required_argument, 0, 'g'},
+	{"help",             no_argument,       0, 'h'},
+	{"foreground",       no_argument,       0, 'i'},
+	{"stdio",            no_argument,       0, 'I'},
+	{"limit",            required_argument, 0, 'l'},
+	{"other",            optional_argument, 0, 'o'},
+	{"port",             required_argument, 0, 'p'},
+	{"quiet",            no_argument,       0, 'q'},
+	{"reply",            required_argument, 0, 'r'},
+	{"reply-all",        required_argument, 0, 'R'},
+	{"nosyslog",         no_argument,       0, 'S'},
+	{"timeout",          required_argument, 0, 't'},
+	{"user",             required_argument, 0, 'u'},
+	{"version",          no_argument,       0, 'v'},
 #if MASQ_SUPPORT
-	{"forward",				optional_argument,	0, 'f'},
-	{"masquerade",				no_argument,		0, 'm'},
-	{"masquerade-first",			no_argument,		0, 'M'},
-	{"forward-last",			no_argument,		0, '0'}, /* deprecated */
+	{"forward",          optional_argument, 0, 'f'},
+	{"masquerade",       no_argument,       0, 'm'},
+	{"masquerade-first", no_argument,       0, 'M'},
 #endif
-	{"proxy",				required_argument,	0, 'P'},
+	{"proxy",            required_argument, 0, 'P'},
 	{NULL, 0, NULL, 0}
 };
 
@@ -209,12 +207,6 @@ int get_options(int argc, char *const argv[]) {
 				enable_opt(MASQ);
 				break;
 
-			case '0':
-				o_log(LOG_CRIT, "Warning: The --forward-last flag is deprecated "
-				                "and will be removed in the future. Please use "
-				                "--masquerade-first (-M) instead.");
-				/* fall through */
-
 			case 'M':
 				enable_opt(MASQ | FORWARD | MASQ_OVERRIDE);
 				break;
@@ -300,6 +292,11 @@ int get_options(int argc, char *const argv[]) {
 				failuser = xstrdup(optarg);
 				break;
 
+			case 'R':
+				free(replyall);
+				replyall = xstrdup(optarg);
+				break;
+
 			case 'S':
 				enable_opt(NOSYSLOG);
 				break;
@@ -324,14 +321,6 @@ int get_options(int argc, char *const argv[]) {
 				}
 				break;
 
-#if HAVE_LIBUDB
-			case 'U':
-				o_log(LOG_CRIT, "Warning: UDB support is deprecated and will "
-				                "be removed in the future.");
-				enable_opt(USEUDB);
-				break;
-#endif
-
 			case 'v':
 				print_version(true);
 				exit(EXIT_SUCCESS);
@@ -365,6 +354,11 @@ int get_options(int argc, char *const argv[]) {
 	}
 
 #if NEED_ROOT
+	if (geteuid() != 0) {
+		o_log(LOG_CRIT, "Fatal: " PACKAGE_NAME " needs to run as root on this system");
+		return -1;
+	}
+
 	/*
 	** Warn the user that privileges will not be dropped automatically.
 	*/
@@ -450,13 +444,9 @@ static void print_usage(void) {
 "-S or --nosyslog             Write messages to stderr instead of syslog\n"
 "-t or --timeout <seconds>    Wait at most <seconds> before closing connections\n"
 "-u or --user <user>          Run as specified user or UID\n"
-
-#if HAVE_LIBUDB
-"-U or --udb                  Perform lookups in UDB shared memory tables (deprecated)\n"
-#endif
-
 "-v or --version              Display version information and exit\n"
 "-r or --reply <string>       If a query fails, pretend it succeeded, returning <string>\n"
+"-R or --reply-all <string>   Always return <string> without performing connection lookups\n"
 "-h or --help                 Display this help and exit\n";
 
 	print_version(false);
@@ -475,7 +465,7 @@ static void print_version(bool verbose) {
 	printf("%s by %s <%s>\n", PACKAGE_STRING,
 			PACKAGE_AUTHOR, PACKAGE_BUGREPORT);
 	printf("Originally written by Ryan McCabe <ryan@numb.org>\n");
-	printf("%s\n", PACKAGE_WEBSITE);
+	printf("%s\n", PACKAGE_URL);
 
 	if (verbose) {
 		printf("\nBuild information:\n");
@@ -486,7 +476,6 @@ static void print_version(bool verbose) {
 		print_version_bool("Masquerading support", MASQ_SUPPORT);
 		print_version_bool("IPv6 support", WANT_IPV6);
 		print_version_bool("Linux libnfct support", LIBNFCT_SUPPORT);
-		print_version_bool("UDB library support", HAVE_LIBUDB);
 
 		printf("\nBuild settings:\n");
 		print_version_str("Configuration directory", SYSCONFDIR);
