@@ -1,7 +1,7 @@
 /*
-** oidentd_options.h - oidentd command-line handler.
+** options.h - oidentd command-line handler.
 ** Copyright (c) 2001-2006 Ryan McCabe <ryan@numb.org>
-** Copyright (c) 2018      Janik Rabe  <oidentd@janikrabe.com>
+** Copyright (c) 2018-2019 Janik Rabe  <info@janikrabe.com>
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License, version 2,
@@ -22,32 +22,23 @@
 
 #define _GNU_SOURCE
 
-#define CHANGE_UID    (1 << 0x01)
-#define CHANGE_GID    (1 << 0x02)
-#define HIDE_ERRORS   (1 << 0x03)
-#define MASQ          (1 << 0x04)
-#define FORWARD       (1 << 0x05)
-#define PROXY         (1 << 0x06)
-#define USEUDB        (1 << 0x07)
-#define DEBUG_MSGS    (1 << 0x08)
-#define QUIET         (1 << 0x09)
-#define FOREGROUND    (1 << 0x0a)
-#define NOSYSLOG      (1 << 0x0b)
-#define STDIO         (1 << 0x0c)
-#define MASQ_OVERRIDE (1 << 0x0d)
+#define CHANGE_UID    (1 << 0x00)
+#define CHANGE_GID    (1 << 0x01)
+#define HIDE_ERRORS   (1 << 0x02)
+#define MASQ          (1 << 0x03)
+#define FORWARD       (1 << 0x04)
+#define PROXY         (1 << 0x05)
+#define DEBUG_MSGS    (1 << 0x06)
+#define QUIET         (1 << 0x07)
+#define FOREGROUND    (1 << 0x08)
+#define NOSYSLOG      (1 << 0x09)
+#define STDIO         (1 << 0x0a)
+#define MASQ_OVERRIDE (1 << 0x0b)
 
 #ifndef LIBNFCT_SUPPORT
 #define LIBNFCT_SUPPORT 0
 #endif
 
-#ifndef HAVE_LIBUDB
-#define HAVE_LIBUDB 0
-#endif
-
-#ifndef HAVE_LIBCAP_NG
-#define HAVE_LIBCAP_NG 0
-#endif
-
 bool opt_enabled(u_int32_t option);
 void disable_opt(u_int32_t option);
 int get_options(int argc, char *const argv[]);
