@@ -1,7 +1,7 @@
 /*
-** oidentd_util.c - oidentd utility functions.
+** util.c - oidentd utility functions.
 ** Copyright (c) 2001-2006 Ryan McCabe <ryan@numb.org>
-** Copyright (c) 2018-2019 Janik Rabe  <oidentd@janikrabe.com>
+** Copyright (c) 2018-2019 Janik Rabe  <info@janikrabe.com>
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License, version 2,
@@ -47,31 +47,94 @@
 #include "missing.h"
 #include "options.h"
 
-#if HAVE_LIBUDB
-#	include <udb.h>
+#ifdef HAVE_ARC4RANDOM_UNIFORM
+	/* no rescale required */
+#elif defined HAVE_LRAND48
+#	define O_RAND_UPPER_EXCL_UL (1UL << 31)
+#else
+#	define O_RAND_UPPER_EXCL_UL ((unsigned long) RAND_MAX + 1UL)
 #endif
 
 /*
-** Seed PRNG from time-of-day clock. (FIXME: Should use entropy pool on
-** systems that have one. Should prefer lrand48()(3), then random()(3)
-** then rand()(3) in that order.)
+** Seed the PRNG.
+** A time-based seed is sufficient as oidentd does not require
+** cryptographically secure random numbers.
 ** Returns 0 on success, -1 on failure.
 */
 
-int random_seed(void) {
-#ifdef FUZZING_BUILD_MODE_UNSAFE_FOR_PRODUCTION
-	srand(0);
-#else
-	struct timeval tv;
+int seed_prng(void) {
+#ifndef HAVE_ARC4RANDOM_UNIFORM
+#	ifndef FUZZING_BUILD_MODE_UNSAFE_FOR_PRODUCTION
+		struct timespec tp;
 
-	if (gettimeofday(&tv, NULL))
-		return -1;
-
-	srand((u_int32_t) (tv.tv_sec ^ tv.tv_usec));
+		if (clock_gettime(CLOCK_REALTIME, &tp)) {
+			debug("clock_gettime: %s", strerror(errno));
+			return -1;
+		}
+#	endif
 #endif
+
+#ifdef HAVE_ARC4RANDOM_UNIFORM
+	/* automatically reseeded upon fork(2) */
+#elif defined HAVE_LRAND48
+#	ifdef FUZZING_BUILD_MODE_UNSAFE_FOR_PRODUCTION
+		srand48(0);
+#	else
+		srand48((long) (tp.tv_sec ^ tp.tv_nsec));
+#	endif
+#elif defined HAVE_RANDOM
+#	ifdef FUZZING_BUILD_MODE_UNSAFE_FOR_PRODUCTION
+		srandom(0);
+#	else
+		srandom((unsigned int) (tp.tv_sec ^ tp.tv_nsec));
+#	endif
+#else
+#	ifdef FUZZING_BUILD_MODE_UNSAFE_FOR_PRODUCTION
+		srand(0);
+#	else
+		srand((unsigned int) (tp.tv_sec ^ tp.tv_nsec));
+#	endif
+#endif
+
 	return 0;
 }
 
+#ifndef HAVE_ARC4RANDOM_UNIFORM
+
+/*
+** Return a pseudorandom integer in the interval [0, O_RAND_UPPER_EXCL_UL).
+*/
+unsigned long prng_next(void) {
+#ifdef HAVE_LRAND48
+	return (unsigned long) lrand48();
+#elif defined HAVE_RANDOM
+	return (unsigned long) random();
+#else
+	return (unsigned long) rand();
+#endif
+}
+#endif
+
+/*
+** Return a pseudorandom integer in the interval [0, i).
+*/
+
+unsigned int randval(unsigned int i) {
+#ifdef HAVE_ARC4RANDOM_UNIFORM
+	return (unsigned int) arc4random_uniform((uint32_t) i);
+#else
+	unsigned long blk_size = O_RAND_UPPER_EXCL_UL / (unsigned long) i;
+	unsigned long blk_after_last = blk_size * (unsigned long) i;
+	unsigned long next;
+
+	do {
+		next = prng_next();
+	} while (next >= blk_after_last);
+
+	return (unsigned int) (next / blk_size);
+#endif
+}
+
 /*
 ** Find the user specified by "temp_user"
 ** Returns non-zero on failure.
@@ -132,12 +195,18 @@ int find_group(const char *temp_group, gid_t *gid) {
 */
 
 int drop_privs(uid_t new_uid, gid_t new_gid) {
-
 	if (opt_enabled(CHANGE_GID)) {
 		if (setgid(new_gid) != 0) {
-			debug("setgid(%u): %s", new_gid, strerror(errno));
+			debug("setgid(%lu): %s", (unsigned long) new_gid, strerror(errno));
 			return -1;
 		}
+
+#ifdef HAVE_SETGROUPS
+		if (setgroups(0, NULL)) {
+			debug("setgroups: %s", strerror(errno));
+			return -1;
+		}
+#endif
 	}
 
 	if (opt_enabled(CHANGE_UID)) {
@@ -147,7 +216,7 @@ int drop_privs(uid_t new_uid, gid_t new_gid) {
 
 		pw = getpwuid(new_uid);
 		if (!pw) {
-			debug("getpwuid(%u): No such user", new_uid);
+			debug("getpwuid(%lu): No such user ID", (unsigned long) new_uid);
 			return -1;
 		}
 
@@ -159,13 +228,13 @@ int drop_privs(uid_t new_uid, gid_t new_gid) {
 		ret = initgroups(pw->pw_name, my_gid);
 
 		if (ret != 0) {
-			debug("initgroups(%s, %u): %s", pw->pw_name, my_gid,
+			debug("initgroups(%s, %lu): %s", pw->pw_name, (unsigned long) my_gid,
 				strerror(errno));
 			return -1;
 		}
 
 		if (setuid(new_uid) != 0) {
-			debug("setuid(%u): %s", new_uid, strerror(errno));
+			debug("setuid(%lu): %s", (unsigned long) new_uid, strerror(errno));
 			return -1;
 		}
 	}
@@ -199,8 +268,8 @@ FILE *safe_open(const struct passwd *pw, const char *filename) {
 
 	if (st.st_uid != pw->pw_uid) {
 		o_log(LOG_CRIT,
-			"Refused to read %s during request for %s (owner is UID %d)",
-			path, pw->pw_name, st.st_uid);
+			"Refused to read %s during request for %s (owner is UID %lu)",
+			path, pw->pw_name, (unsigned long) st.st_uid);
 
 		goto out_fail;
 	}
@@ -208,7 +277,7 @@ FILE *safe_open(const struct passwd *pw, const char *filename) {
 	fp = fopen(path, "r");
 	if (!fp) {
 		if (errno != ENOENT)
-			debug("open: %s: : %s", path, strerror(errno));
+			debug("open: %s: %s", path, strerror(errno));
 
 		goto out_fail;
 	}
@@ -222,8 +291,8 @@ FILE *safe_open(const struct passwd *pw, const char *filename) {
 
 	if (st.st_uid != pw->pw_uid) {
 		o_log(LOG_CRIT,
-			"Refused to read \"%s\" during request for %s (owner is UID %u)",
-			path, pw->pw_name, st.st_uid);
+			"Refused to read %s during request for %s (owner is UID %lu)",
+			path, pw->pw_name, (unsigned long) st.st_uid);
 
 		fclose(fp);
 		goto out_fail;
@@ -250,7 +319,7 @@ int go_background(void) {
 		case 0:
 			break;
 		default:
-			_exit(0);
+			_exit(EXIT_SUCCESS);
 	}
 
 	if (setsid() == (pid_t) -1) {
@@ -408,7 +477,7 @@ int o_log(int priority, const char *fmt, ...) {
 	if (opt_enabled(QUIET) && priority != LOG_CRIT)
 		return 0;
 
-	if (priority == DEBUG && !opt_enabled(DEBUG_MSGS))
+	if (priority == LOG_DEBUG && !opt_enabled(DEBUG_MSGS))
 		return 0;
 
 	va_start(ap, fmt);
@@ -423,69 +492,3 @@ int o_log(int priority, const char *fmt, ...) {
 	free(buf);
 	return ret;
 }
-
-#if HAVE_LIBUDB
-/*
-** Look up a connection in the UDB shared memory tables.
-**
-** This only supports IPv4 right now.
-*/
-
-struct udb_lookup_res get_udb_user(	in_port_t lport,
-					in_port_t fport,
-					const struct sockaddr_storage *laddr,
-					const struct sockaddr_storage *faddr,
-					int sock)
-{
-	struct udb_connection conn;
-	struct udb_conn_user buf;
-	struct udb_lookup_res res = {0, (uid_t) -1};
-	struct passwd *pw;
-	char faddr_buf[MAX_IPLEN];
-	char laddr_buf[MAX_IPLEN];
-	extern char *ret_os;
-
-	if (laddr->ss_family != AF_INET || faddr->ss_family != AF_INET)
-		return res;
-
-	memset(&conn, 0, sizeof(conn));
-
-	conn.from.sin_family = laddr->ss_family;
-	memcpy(&conn.from.sin_addr, &SIN4(laddr)->sin_addr,
-		sizeof(conn.from.sin_addr));
-	conn.from.sin_port = htons(lport);
-
-	conn.to.sin_family = faddr->ss_family;
-	memcpy(&conn.to.sin_addr, &SIN4(faddr)->sin_addr, sizeof(conn.to.sin_addr));
-	conn.to.sin_port = htons(fport);
-
-	get_ip(faddr, faddr_buf, sizeof(faddr_buf));
-	get_ip(laddr, laddr_buf, sizeof(laddr_buf));
-
-	debug("UDB lookup: %s:%d->%s:%d",
-		laddr_buf, ntohs(conn.from.sin_port),
-		faddr_buf, ntohs(conn.to.sin_port));
-
-	if (!udb_conn_get(&conn, &buf))
-		return res;
-
-	/* If the user is local, return their UID */
-	pw = getpwnam(buf.username);
-	if (pw) {
-		res.status = 1;
-		res.uid = pw->pw_uid;
-		return res;
-	}
-
-	/* User not local, reply with string from UDB table. */
-	sockprintf(sock, "%d,%d:USERID:%s:%s\r\n",
-		lport, fport, ret_os, buf.username);
-
-	o_log(NORMAL, "[%s] UDB lookup: %d , %d : (returned %s)",
-		faddr_buf, lport, fport, buf.username);
-
-	res.status = 2;
-	return res;
-}
-
-#endif
